==> xorp-1.1.tar.gz: xorp-1.1/libxorp/test_heap.cc <==
// -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-

// Copyright (c) 2001-2005 International Computer Science Institute
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software")
// to deal in the Software without restriction, subject to the conditions
// listed in the XORP LICENSE file. These conditions include: you must
// preserve this copyright notice, and you cannot mention the copyright
// holders in advertising related to the Software without their permission.
// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
// notice is a summary of the XORP LICENSE file; the license in that file is
// legally binding.

#ident "$XORP: xorp/libxorp/test_heap.cc,v 1.2 2005/03/25 02:53:45 pavlin Exp $"

#include "libxorp_module.h"
#include "libxorp/xorp.h"
#include "libxorp/xlog.h"
#include "libxorp/exceptions.hh"
#include "libxorp/heap.hh"


//
// XXX: MODIFY FOR YOUR TEST PROGRAM
//
static const char *program_name		= "test_heap";
static const char *program_description	= "Test Heap class";
static const char *program_version_id	= "0.1";
static const char *program_date		= "July 12, 2004";
static const char *program_copyright	= "See file LICENSE.XORP";
static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";

static bool s_verbose = false;
bool verbose()			{ return s_verbose; }
void set_verbose(bool v)	{ s_verbose = v; }

static int s_failures = 0;
bool failures()			{ return s_failures; }
void incr_failures()		{ s_failures++; }



//
// printf(3)-like facility to conditionally print a message if verbosity
// is enabled.
//
#define verbose_log(x...) _verbose_log(__FILE__,__LINE__, x)

#define _verbose_log(file, line, x...)					\
do {									\
    if (verbose()) {							\
	printf("From %s:%d: ", file, line);				\
	printf(x);							\
    }									\
} while(0)


//
// Test and print a message whether two strings are lexicographically same.
// The strings can be either C or C++ style.
//
#define verbose_match(s1, s2)						\
    _verbose_match(__FILE__, __LINE__, s1, s2)

bool
_verbose_match(const char* file, int line, const string& s1, const string& s2)
{
    bool match = s1 == s2;

    _verbose_log(file, line, "Comparing %s == %s : %s\n",
		 s1.c_str(), s2.c_str(), match ? "OK" : "FAIL");
    if (match == false)
	incr_failures();
    return match;
}


//
// Test and print a message whether a condition is true.
//
// The first argument is the condition to test.
// The second argument is a string with a brief description of the tested
// condition.
//
#define verbose_assert(cond, desc) 					\
    _verbose_assert(__FILE__, __LINE__, cond, desc)

bool
_verbose_assert(const char* file, int line, bool cond, const string& desc)
{
    _verbose_log(file, line,
		 "Testing %s : %s\n", desc.c_str(), cond ? "OK" : "FAIL");
    if (cond == false)
	incr_failures();
    return cond;
}


/**
 * Print program info to output stream.
 *
 * @param stream the output stream the print the program info to.
 */
static void
print_program_info(FILE *stream)
{
    fprintf(stream, "Name:          %s\n", program_name);
    fprintf(stream, "Description:   %s\n", program_description);
    fprintf(stream, "Version:       %s\n", program_version_id);
    fprintf(stream, "Date:          %s\n", program_date);
    fprintf(stream, "Copyright:     %s\n", program_copyright);
    fprintf(stream, "Return:        %s\n", program_return_value);
}

/**
 * Print program usage information to the stderr.
 *
 * @param progname the name of the program.
 */
static void
usage(const char* progname)
{
    print_program_info(stderr);
    fprintf(stderr, "usage: %s [-v] [-h]\n", progname);
    fprintf(stderr, "       -h          : usage (this message)\n");
    fprintf(stderr, "       -v          : verbose output\n");
    fprintf(stderr, "Return 0 on success, 1 if test error, 2 if internal error.\n");
}

class TestHeap : public Heap {
public:
    TestHeap() : Heap(0) {}

    void test_heap_push();
    void test_heap_push_same_value();
};

/**
 * Test Heap valid constructors.
 */
void
test_heap_valid_constructors()
{

}

/**
 * Test Heap invalid constructors.
 */
void
test_heap_invalid_constructors()
{

}

void
TestHeap::test_heap_push()
{
    struct heap_entry* he;
    int i1 = 1;
    int i2 = 2;
    int i3 = 3;

    push(TimeVal(0, i1), &i1);
    push(TimeVal(0, i2), &i2);
    push(TimeVal(0, i3), &i3);

    verbose_assert(size() == 3, "heap size");
    he = top();
    verbose_assert(he->object == &i1, "heap top value 1");

    pop();
    verbose_assert(size() == 2, "heap size");
    he = top();
    verbose_assert(he->object == &i2, "heap top value 2");

    pop();
    verbose_assert(size() == 1, "heap size");
    he = top();
    verbose_assert(he->object == &i3, "heap top value 3");

    pop();
    verbose_assert(size() == 0, "heap size");
}

void
TestHeap::test_heap_push_same_value()
{
    struct heap_entry* he;
    int i1_1 = 1;
    int i1_2 = 1;

    push(TimeVal(0, i1_1), &i1_1);
    push(TimeVal(0, i1_2), &i1_2);

    verbose_assert(size() == 2, "heap size");
    he = top();
    verbose_assert(he->object == &i1_1, "heap top value 1_1");

    pop();
    verbose_assert(size() == 1, "heap size");
    he = top();
    verbose_assert(he->object == &i1_2, "Test heap top value 1_2");
    pop();
    verbose_assert(size() == 0, "heap size");
}

int
main(int argc, char * const argv[])
{
    int ret_value = 0;

    //
    // Initialize and start xlog
    //
    xlog_init(argv[0], NULL);
    xlog_set_verbose(XLOG_VERBOSE_LOW);         // Least verbose messages
    // XXX: verbosity of the error messages temporary increased
    xlog_level_set_verbose(XLOG_LEVEL_ERROR, XLOG_VERBOSE_HIGH);
    xlog_add_default_output();
    xlog_start();

    int ch;
    while ((ch = getopt(argc, argv, "hv")) != -1) {
	switch (ch) {
	case 'v':
	    set_verbose(true);
	    break;
	case 'h':
	case '?':
	default:
	    usage(argv[0]);
	    xlog_stop();
	    xlog_exit();
	    if (ch == 'h')
		return (0);
	    else
		return (1);
	}
    }
    argc -= optind;
    argv += optind;

    XorpUnexpectedHandler x(xorp_unexpected_handler);
    try {
	TestHeap heap;
	test_heap_valid_constructors();
	test_heap_invalid_constructors();
	heap.test_heap_push();
	heap.test_heap_push_same_value();
	ret_value = failures() ? 1 : 0;
    } catch (...) {
	// Internal error
	xorp_print_standard_exceptions();
	ret_value = 2;
    }

    //
    // Gracefully stop and exit xlog
    //
    xlog_stop();
    xlog_exit();

    return (ret_value);
}

==> xorp-1.3.tar.gz: xorp-1.3/libxorp/test_heap.cc <==
// -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-

// Copyright (c) 2001-2006 International Computer Science Institute
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software")
// to deal in the Software without restriction, subject to the conditions
// listed in the XORP LICENSE file. These conditions include: you must
// preserve this copyright notice, and you cannot mention the copyright
// holders in advertising related to the Software without their permission.
// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
// notice is a summary of the XORP LICENSE file; the license in that file is
// legally binding.

#ident "$XORP: xorp/libxorp/test_heap.cc,v 1.6 2006/03/16 00:04:33 pavlin Exp $"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "libxorp_module.h"
#include "libxorp/xorp.h"
#include "libxorp/xlog.h"
#include "libxorp/exceptions.hh"
#include "libxorp/heap.hh"

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

//
// XXX: MODIFY FOR YOUR TEST PROGRAM
//
static const char *program_name		= "test_heap";
static const char *program_description	= "Test Heap class";
static const char *program_version_id	= "0.1";
static const char *program_date		= "July 12, 2004";
static const char *program_copyright	= "See file LICENSE.XORP";
static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";

static bool s_verbose = false;
bool verbose()			{ return s_verbose; }
void set_verbose(bool v)	{ s_verbose = v; }

static int s_failures = 0;
bool failures()			{ return s_failures; }
void incr_failures()		{ s_failures++; }



//
// printf(3)-like facility to conditionally print a message if verbosity
// is enabled.
//
#define verbose_log(x...) _verbose_log(__FILE__,__LINE__, x)

#define _verbose_log(file, line, x...)					\
do {									\
    if (verbose()) {							\
	printf("From %s:%d: ", file, line);				\
	printf(x);							\
    }									\
} while(0)


//
// Test and print a message whether two strings are lexicographically same.
// The strings can be either C or C++ style.
//
#define verbose_match(s1, s2)						\
    _verbose_match(__FILE__, __LINE__, s1, s2)

bool
_verbose_match(const char* file, int line, const string& s1, const string& s2)
{
    bool match = s1 == s2;

    _verbose_log(file, line, "Comparing %s == %s : %s\n",
		 s1.c_str(), s2.c_str(), match ? "OK" : "FAIL");
    if (match == false)
	incr_failures();
    return match;
}


//
// Test and print a message whether a condition is true.
//
// The first argument is the condition to test.
// The second argument is a string with a brief description of the tested
// condition.
//
#define verbose_assert(cond, desc) 					\
    _verbose_assert(__FILE__, __LINE__, cond, desc)

bool
_verbose_assert(const char* file, int line, bool cond, const string& desc)
{
    _verbose_log(file, line,
		 "Testing %s : %s\n", desc.c_str(), cond ? "OK" : "FAIL");
    if (cond == false)
	incr_failures();
    return cond;
}


/**
 * Print program info to output stream.
 *
 * @param stream the output stream the print the program info to.
 */
static void
print_program_info(FILE *stream)
{
    fprintf(stream, "Name:          %s\n", program_name);
    fprintf(stream, "Description:   %s\n", program_description);
    fprintf(stream, "Version:       %s\n", program_version_id);
    fprintf(stream, "Date:          %s\n", program_date);
    fprintf(stream, "Copyright:     %s\n", program_copyright);
    fprintf(stream, "Return:        %s\n", program_return_value);
}

/**
 * Print program usage information to the stderr.
 *
 * @param progname the name of the program.
 */
static void
usage(const char* progname)
{
    print_program_info(stderr);
    fprintf(stderr, "usage: %s [-v] [-h]\n", progname);
    fprintf(stderr, "       -h          : usage (this message)\n");
    fprintf(stderr, "       -v          : verbose output\n");
    fprintf(stderr, "Return 0 on success, 1 if test error, 2 if internal error.\n");
}

class TestHeap : public Heap {
public:
    TestHeap() : Heap(0) {}

    void test_heap_push();
    void test_heap_push_same_value();
};

/**
 * Test Heap valid constructors.
 */
void
test_heap_valid_constructors()
{

}

/**
 * Test Heap invalid constructors.
 */
void
test_heap_invalid_constructors()
{

}

void
TestHeap::test_heap_push()
{
    struct heap_entry* he;
    int i1 = 1;
    int i2 = 2;
    int i3 = 3;

    push(TimeVal(0, i1), reinterpret_cast<HeapBase *>(&i1));
    push(TimeVal(0, i2), reinterpret_cast<HeapBase *>(&i2));
    push(TimeVal(0, i3), reinterpret_cast<HeapBase *>(&i3));

    verbose_assert(size() == 3, "heap size");
    he = top();
    verbose_assert(he->object == reinterpret_cast<HeapBase *>(&i1),
		   "heap top value 1");

    pop();
    verbose_assert(size() == 2, "heap size");
    he = top();
    verbose_assert(he->object == reinterpret_cast<HeapBase *>(&i2),
		   "heap top value 2");

    pop();
    verbose_assert(size() == 1, "heap size");
    he = top();
    verbose_assert(he->object == reinterpret_cast<HeapBase *>(&i3),
		   "heap top value 3");

    pop();
    verbose_assert(size() == 0, "heap size");
}

void
TestHeap::test_heap_push_same_value()
{
    struct heap_entry* he;
    int i1_1 = 1;
    int i1_2 = 1;

    push(TimeVal(0, i1_1), reinterpret_cast<HeapBase *>(&i1_1));
    push(TimeVal(0, i1_2), reinterpret_cast<HeapBase *>(&i1_2));

    verbose_assert(size() == 2, "heap size");
    he = top();
    verbose_assert(he->object == reinterpret_cast<HeapBase *>(&i1_1),
		   "heap top value 1_1");

    pop();
    verbose_assert(size() == 1, "heap size");
    he = top();
    verbose_assert(he->object == reinterpret_cast<HeapBase *>(&i1_2),
		   "Test heap top value 1_2");
    pop();
    verbose_assert(size() == 0, "heap size");
}

int
main(int argc, char * const argv[])
{
    int ret_value = 0;

    //
    // Initialize and start xlog
    //
    xlog_init(argv[0], NULL);
    xlog_set_verbose(XLOG_VERBOSE_LOW);         // Least verbose messages
    // XXX: verbosity of the error messages temporary increased
    xlog_level_set_verbose(XLOG_LEVEL_ERROR, XLOG_VERBOSE_HIGH);
    xlog_add_default_output();
    xlog_start();

    int ch;
    while ((ch = getopt(argc, argv, "hv")) != -1) {
	switch (ch) {
	case 'v':
	    set_verbose(true);
	    break;
	case 'h':
	case '?':
	default:
	    usage(argv[0]);
	    xlog_stop();
	    xlog_exit();
	    if (ch == 'h')
		return (0);
	    else
		return (1);
	}
    }
    argc -= optind;
    argv += optind;

    XorpUnexpectedHandler x(xorp_unexpected_handler);
    try {
	TestHeap heap;
	test_heap_valid_constructors();
	test_heap_invalid_constructors();
	heap.test_heap_push();
	heap.test_heap_push_same_value();
	ret_value = failures() ? 1 : 0;
    } catch (...) {
	// Internal error
	xorp_print_standard_exceptions();
	ret_value = 2;
    }

    //
    // Gracefully stop and exit xlog
    //
    xlog_stop();
    xlog_exit();

    return (ret_value);
}
